/* A class for dealing with .NFO files */

#include "info.h"
#include "sprites.h"

const char *infoline = "%s %d %d %02X %d %d %d %d";
//		<PCX-File> <X> <Y> <info[0..7]>
//		extended if info[0]&8: info[1]*<%d %d> linelen linestart


int makeint(U8 low, S8 high)
{
  S16 combined;

  combined = (high << 8) | low;
  return combined;
}


inforeader::inforeader(FILE *fn)
{
  f = fn;

  pcx = NULL;
  strcpy(lastpcxname, ":\\");	// invalid file name -> no file open so far

  fseek(f, 0, SEEK_END);
  filesize = ftell(f);
  fseek(f, 0, SEEK_SET);

  fgets(pcxname, 128, f);	// skip first line, a comment
  fgets(pcxname, 128, f);	// second line used to get info version
  if (strncmp(pcxname, "// (Info version ", 17)) {
	infover = 1;
	fseek(f, 0, SEEK_SET);
	fgets(pcxname, 128, f);
  } else
	infover = atoi(pcxname + 17);

  if (infover > 2)
	fgets(pcxname, 128, f);	// skip "format: " line

  lasty = -1;
}

inforeader::~inforeader()
{
  fclose(f);
}

int inforeader::next(int wantno)
{
  int i;

  if (feof(f)) return 0;

  if (infover >= 2)
	fscanf(f, "%5d ", &i);
  else
	i = wantno;

  fscanf(f, "%s", pcxname);
  if ( (strlen(pcxname) == 0) || feof(f) )
	return 0;

  if (i != wantno) {
	printf("Error: Sprite number mismatch, found %d but wanted %d\n", i, wantno);
	exit(2);
  }

  if (strcmp(pcxname, "*") == 0) {
	verbatim = 1;
	fscanf(f, "%d ", &size);	// read how many bytes
  } else {
	verbatim = 0;
	if ( (pcx == NULL) || (stricmp(pcxname, lastpcxname) != 0) ) {
		if (pcx)
			delete(pcx);
		pcx = new pcxread(new singlefile(pcxname, "rb"));
		if (!pcx) {
			printf("Error: can't open %s\n", pcxname);
			exit(2);
		}

		strcpy(lastpcxname, pcxname);
		pcx->startimage(0, 0, 0, 0, NULL);
	}

	int x, y;

	if (infover<3) {
		fscanf(f, "%d %d %x %x %x %x %x %x %x %x",
			&x, &y,
			&(intinf[0]), &(intinf[1]), &(intinf[2]), &(intinf[3]),
			&(intinf[4]), &(intinf[5]), &(intinf[6]), &(intinf[7]));
	} else {
		int rx, ry;
		fscanf(f, "%d %d %x %d %d %d %d",
			&x, &y,
			&(intinf[0]), &(intinf[1]),
			&sx, &rx, &ry);
		intinf[2] = sx & 0xff;
		intinf[3] = sx >> 8;
		intinf[4] = rx & 0xff;
		intinf[5] = rx >> 8;
		intinf[6] = ry & 0xff;
		intinf[7] = ry >> 8;
	}

	if (y < lasty) {
		printf("Error: y is %d, smaller than %d. Y can't decrease!\n", y, lasty);
		exit(2);
	}

	sx = (intinf[3] << 8) | intinf[2];
	sy = intinf[1];

	for (int i=0; i<8; i++)
		inf[i] = intinf[i];

	if (infover < 4)
		y++;	// bug, had an extra line at the top

	pcx->startsubimage(x, y, sx, sy);

	imgsize = (long) sx * (long) sy;

	do {
		fgets(pcxname, 128, f);
	} while (!strrchr(pcxname, '\n'));
  }

  return 1;
}

int inforeader::nextverb()
{
  int byte;
  fscanf(f, "%2x ", &byte);
  return byte;
}

int inforeader::getsprite(U8 *sprite)
{
  pcx->streamgetpixel(sprite, imgsize);
  return 1;
}


infowriter::infowriter(FILE *info, int maxboxes)
{
  infowriter::info = info;
  fputs("// Automatically generated by GRFCODEC. Do not modify!\n", info);
  fputs("// (Info version 4)\n", info);
  fputs("// Format: spritenum pcxfile xpos ypos compression ysize xsize xrel yrel\n", info);

  infowriter::maxboxes = maxboxes;
  boxes = new box[maxboxes];
  boxnum = 0;
  spriteno = 0;
  for (int i=0; i<maxboxes; i++)
	boxes[i].type = isinvalid;
}

void infowriter::newband(pcxfile *pcx)
{
  int i;
  U16 j;

  for (i=0; i<boxnum; i++) {
	fprintf(info, "%5d ", spriteno++);
	switch (boxes[i].type) {
		case issprite: {
			box::boxsprite *s = &(boxes[i].sprite);

			fprintf(info, infoline, pcx->filename(), s->x,
				pcx->subimagey(),
				s->info[0], s->info[1],
				makeint(s->info[2], s->info[3]),
				makeint(s->info[4], s->info[5]),
				makeint(s->info[6], s->info[7]));
			fputs("\n", info);
			break;  }

		case isdata:	{
			box::boxdata *d = &(boxes[i].data);

			fprintf(info, "* %d\t", d->size);
			for (j=0; j<d->size; j++) {
				fprintf(info, " %02X", d->data[j]);
				if ( ( (j & 31) == 31) && (j<d->size-1) )
					fputs("\n\t", info);
			}

			fputs("\n", info);
			delete(d->data);
			break;	}

		default:
			printf("Huh? What kind of box is that?\n");
			exit(2);
	}
  }

  boxnum = 0;

  for (i=0; i<maxboxes; i++)
	boxes[i].type = isinvalid;
}

void infowriter::resize(int newmaxboxes)
{
  printf("Reallocating memory for %d boxes\n", newmaxboxes);
  infowriter::maxboxes = newmaxboxes;
  box *newboxes = new box[newmaxboxes];
  int i;
  for (i=0; i<maxboxes; i++)
	newboxes[i] = boxes[i];
  for (; i<newmaxboxes; i++)
	newboxes[i].type = isinvalid;
  delete(boxes);
  boxes = newboxes;
}

void infowriter::addsprite(int x, U8 info[8])
{
  if (boxnum >= maxboxes)
	resize(maxboxes*2);

  boxes[boxnum].type = issprite;
  boxes[boxnum].sprite.x = x;
  memcpy(boxes[boxnum].sprite.info, info, 8*sizeof(U8));

  boxnum++;
}

void infowriter::adddata(U16 size, U8 *data)
{
  if (boxnum >= maxboxes)
	resize(maxboxes*2);

  boxes[boxnum].type = isdata;
  boxes[boxnum].data.size = size;
  boxes[boxnum].data.data = data;

  boxnum++;
}

void infowriter::done(int count)
{
  if (count != spriteno) {
	printf("Error: GRFCodec thinks it has written %d sprites but the info file only got %d\n", count, spriteno);
	exit(2);
  }
}


